<?php
/**
 * MG Main manager.
 *
 * @package MG
 * @since   1.0
 */
if ( ! function_exists( 'MG' ) ) {
	/**
	 * MG manager.
	 * @since 1.0
	 * @return MG
	 */
	function MG() {
		global $MG;

		return $MG;
	}
}

if ( ! function_exists( 'MG_Settings' ) ) {
	/**
	 * MG manager.
	 * @since 1.0
	 * @return MG_Settings
	 */
	function MG_Settings() {
		return new MG_Settings();
	}
}

if ( ! function_exists( 'MG_Social' ) ) {
	/**
	 * MG manager.
	 * @since 1.0
	 * @return MG_Social
	 */
	function MG_Social() {
		return new MG_Social();
	}
}

if ( ! function_exists( 'MG_Settings_Build_Options' ) ) {
	/**
	 * MG manager.
	 * @since 1.0
	 * @return MG_Settings
	 */
	function MG_Settings_Build_Options( $options=array(), $box_id='' ) {
		return MG_Settings()->build_options( $options, $box_id );
	}
}

/*
if ( ! function_exists( 'MG_Envato_Helper' ) ) {
	function MG_Envato_Helper() {
		return new MG_Envato_Helper();
	}
}*/

if ( ! function_exists( 'MG_Woocommerce' ) ) {
	/**
	 * MG manager.
	 * @since 1.0
	 * @return MG_Settings
	 */
	function MG_Woocommerce() {
		return new MG_Woocommerce();
	}
}

if ( ! function_exists( 'MG_Load_Options' ) ) {
	/**
	 * MG manager.
	 * @since 1.0
	 * @return MG_Settings
	 */
	function MG_Load_Options( $dir ) {
	    new MG_Settings(); // save settings first!
	    
		$options = array();
			
		// find if we have a options.php into the same folder
		$options_file = $dir . '/options.php';
		if( is_file($options_file) ){
			
			ob_start();
			require_once $options_file;
			
			$content = ob_get_contents();
			ob_clean();
 
			if( trim($content) != "" ){
				$options = json_decode( $content, true );
			}
		}
		return $options;
	}
}

if ( ! function_exists( 'MG_mapper' ) ) {
	/**
	 * Shorthand for MG Mapper.
	 * @since 1.0
	 * @return MG_Mapper
	 */
	function MG_mapper() {
		return MG()->mapper();
	}
}
if ( ! function_exists( 'MG_settings' ) ) {
	/**
	 * Shorthand for MG settings.
	 * @since 1.0
	 * @return MG_Settings
	 */
	function MG_settings() {
		return MG()->settings();
	}
}
if ( ! function_exists( 'MG_path_dir' ) ) {
	/**
	 * Get file/directory path in MG.
	 *
	 * @param string $name - path name
	 * @param string $file
	 *
	 * @since 1.0
	 * @return string
	 */
	function MG_path_dir( $name, $file = '' ) {
		return MG()->path( $name, $file );
	}
}
if ( ! function_exists( 'MG_upload_dir' ) ) {
	/**
	 * Temporary files upload dir;
	 * @since 1.0
	 * @return string
	 */
	function MG_upload_dir() {
		return MG()->uploadDir();
	}
}
if ( ! function_exists( 'MG_path_dir' ) ) {
	/**
	 * Get file/directory path in MG.
	 *
	 * @param string $name - path name
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function MG_path_dir( $name, $file = '' ) {
		return MG()->path( $name, $file );
	}
}
if ( ! function_exists( 'MG_post_param' ) ) {
	/**
	 * Get param value from $_POST if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 1.0
	 * @return null|string - null for undefined param.
	 */
	function MG_post_param( $param, $default = null ) {
		return isset( $_POST[ $param ] ) ? $_POST[ $param ] : $default;
	}
}
if ( ! function_exists( 'MG_get_param' ) ) {
	/**
	 * Get param value from $_GET if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 1.0
	 * @return null|string - null for undefined param.
	 */
	function MG_get_param( $param, $default = null ) {
		return isset( $_GET[ $param ] ) ? $_GET[ $param ] : $default;
	}
}
if ( ! function_exists( 'MG_request_param' ) ) {
	/**
	 * Get param value from $_REQUEST if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 4.4
	 * @return null|string - null for undefined param.
	 */
	function MG_request_param( $param, $default = null ) {
		return isset( $_REQUEST[ $param ] ) ? $_REQUEST[ $param ] : $default;
	}
}
if ( ! function_exists( 'MG_action' ) ) {
	/**
	 * Get MG special action param.
	 * @since 1.0
	 * @return string|null
	 */
	function MG_action() {
		$MG_action = 'dashboard';
		if ( isset( $_GET['MG_action'] ) ) {
			$MG_action = $_GET['MG_action'];
		} elseif ( isset( $_POST['MG_action'] ) ) {
			$MG_action = $_POST['MG_action'];
		}

		return $MG_action;
	}
}

if ( ! function_exists( 'MG_asset_url' ) ) {
	/**
	 * Get full url for assets.
	 *
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function MG_asset_url( $file ) {
		return MG()->assetUrl( $file );
	}
}

if ( ! function_exists( 'MG_templates_url' ) ) {
	/**
	 * Get full url for assets.
	 *
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function MG_templates_url( $file ) {
		return MG()->templatesUrl( $file );
	}
}

if ( ! function_exists( 'MG_locate_template' ) ) {
	function MG_locate_template( $template_name, $template_path = '', $default_path = '' ) 
	{
	    if ( ! $template_path ) {
	        $template_path = 'templates/';
	    }

	    if ( ! $default_path ) {
	        $default_path = MG()->path( 'TEMPLATES_DIR', '' );
	    }
 
	    // Look within passed path within the theme - this is priority.
	    $template = locate_template(
	        array(
	            trailingslashit( $template_path ) . $template_name,
	            $template_name
	        )
	    );

	    // Get default template/
	    if ( ! $template ) {
	        $template = $default_path . $template_name;
	    }

	    // Return what we found.
	    return apply_filters( 'MG_locate_template', $template, $template_name, $template_path );
	}
}


if ( ! function_exists( 'MG_Get_IP' ) ) {
	function MG_Get_IP() 
	{
		//Just get the headers if we can or else use the SERVER global
		if ( function_exists( 'apache_request_headers' ) ) {
			$headers = apache_request_headers();
		} else {
			$headers = $_SERVER;
		}
		//Get the forwarded IP if it exists
		if ( array_key_exists( 'X-Forwarded-For', $headers ) && filter_var( $headers['X-Forwarded-For'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 ) ) {
			$the_ip = $headers['X-Forwarded-For'];
		} elseif ( array_key_exists( 'HTTP_X_FORWARDED_FOR', $headers ) && filter_var( $headers['HTTP_X_FORWARDED_FOR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 )
		) {
			$the_ip = $headers['HTTP_X_FORWARDED_FOR'];
		} else {
			$the_ip = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
		}
		return $the_ip;
	}
}

/**
 * Plugin name for MG_time_elapsed_string.
 *
 * @since 1.0
 * @return string
 */
if ( ! function_exists( 'MG_time_elapsed_string' ) ) {
	function MG_time_elapsed_string($ptime)
	{
	    $etime = time() - $ptime;
	
	    if ($etime < 1)
	    {
	        return '0 seconds';
	    }
	
	    $a = array( 365 * 24 * 60 * 60  =>  'year',
	                 30 * 24 * 60 * 60  =>  'month',
	                      24 * 60 * 60  =>  'day',
	                           60 * 60  =>  'hour',
	                                60  =>  'minute',
	                                 1  =>  'second'
	                );
	    $a_plural = array( 'year'   => 'years',
	                       'month'  => 'months',
	                       'day'    => 'days',
	                       'hour'   => 'hours',
	                       'minute' => 'minutes',
	                       'second' => 'seconds'
	                );
	
	    foreach ($a as $secs => $str)
	    {
	        $d = $etime / $secs;
	        if ($d >= 1)
	        {
	            $r = round($d);
	            return $r . ' ' . ($r > 1 ? $a_plural[$str] : $str) . ' ago';
	        }
	    }
	}
}


/**
 * Plugin name for MG.
 *
 * @since 1.0
 * @return string
 */
function MG_plugin_name() {
	return MG()->pluginName();
}