<?php 

/*
 * Define class MG_Template
 * Make sure you skip down to the end of this file, as there are a few
 * lines of code that are very important.
 */
!defined('ABSPATH') and exit;

if (class_exists('MG_Template') != true) {
    
    class MG_Template
    {
        /*
         * Some required plugin information
         */
        const VERSION = '1.0';
        
        public $post_type = '';
        public $cfg = array();
        public $template_directory = '';
        public $the_plugin = array();

        /*
         * Required __construct() function that initalizes the MG_Template
         */
        public function __construct( $the_plugin=array() )
        {

            $this->the_plugin = $the_plugin;

            //add_filter( 'template_include', array( $this, 'template_loader' ) );
            add_shortcode( 'meme_generator_page', array( $this, 'generator_page' ) );

            $this->cfg['fonts'] = array();
            if( isset($this->the_plugin->settings['template_typography']['watermark']['family']) ){
                $this->cfg['fonts'][] = $this->the_plugin->settings['template_typography']['watermark']['family'];
            }

            add_action( 'wp_ajax_mg_ajax_action', array( $this, 'make_request' ) );
            add_action( 'wp_ajax_nopriv_mg_ajax_action', array( $this, 'make_request' ) );

            add_shortcode( 'meme_generator_post', array( $this, 'post_page' ) );
            add_shortcode( 'meme_generator_archive', array( $this, 'archive' ) );

            //add_filter( 'wp_head', array( $this, 'init' ) );

            add_action( 'pre_get_posts', array( $this, 'custom_post_author_archive' ) );
        }

        public function init()
        {
            global $post;

            $suppress_title = false;
            if( is_post_type_archive( 'generated_memes') ) {
                $suppress_title = true;
            }
            if( is_singular( 'generated_memes' ) ){
                $suppress_title = true;
            } 
            if( is_author() === true && $post->post_type == 'generated_memes' ){
                $suppress_title = true;
            }

            if( $suppress_title ){
                //add_filter( 'the_title', array( $this, 'suppress_title' ), 10, 2 );
            }
        }

        public function custom_post_author_archive($query) 
        {
            if ($query->is_author){   
                $query->set( 'post_type', array('generated_memes', 'post') );
            }

            remove_action( 'pre_get_posts', array( $this, 'custom_post_author_archive' ) );
        }

        public function fonts_url() 
        {
            $font_urls = array();
            
            /*
            Translators: If there are characters in your language that are not supported
            by chosen font(s), translate this to 'off'. Do not translate into your own language.
             */
            if ( 'off' !== esc_html_x( 'on', 'Google font: on or off', 'omb' ) ) {
                foreach ($this->cfg['fonts'] as $font_key => $font) {
                    $_fontname = explode( ":", $font );
                    $fontname = $_fontname[0];

                    $new_font = get_theme_mod( $font_key );

                    if( trim($new_font) != "" ){
                        $fontname = str_replace( $fontname, $new_font, $fontname );
                        $font = $fontname .':'. $_fontname[1];
                    }
                    
                    $font_urls[] = $font;
                }
            }
            return add_query_arg( 'family', urlencode( implode('|', array_unique($font_urls)) ), "//fonts.googleapis.com/css");
        }

        public function load_styles_and_script()
        {
            // load frontend google fonts
            if( count($this->cfg['fonts']) > 0 ){
                foreach ($this->cfg['fonts'] as $font) {
                    if( strlen($font) > 5 ){
                        wp_enqueue_style( MG()->alias . '-font-' . $font, '//fonts.googleapis.com/css?family=' . $font, array() );
                    }
                }
            }

            wp_enqueue_style( 'MG-font-awesome', MG_templates_url( 'css/font-awesome.min.css' ), array(), MG_VERSION );
            wp_enqueue_style( 'MG-owl-carousel', MG_templates_url( 'css/owl.carousel.css' ), array(), MG_VERSION );
            //wp_enqueue_style( 'MG-main-style', MG_templates_url( 'css/style.css' ), array(), MG_VERSION );
            wp_enqueue_style( 'MG-frontend_style', admin_url('admin-ajax.php?action=mg_ajax_action&sub_action=frontend_style'), array(), MG_VERSION );

            wp_enqueue_script( MG()->alias . '-owl-carousel', MG_templates_url( 'js/owl.carousel.min.js' ), array('jquery'), '1.0', true);
            wp_enqueue_script( MG()->alias . '-jquery-ui-min', MG_templates_url( 'js/jquery-ui.min.js' ), array('jquery'), '1.0', true);
            wp_enqueue_script( MG()->alias . '-app-class', MG_templates_url( 'js/app.class.js' ), array('jquery'), '1.0', true);
        }

        public function get_meme_templates()
        {
            return get_posts( array(
                'numberposts'   => -1,
                'post_type'     => 'memes_templates',
                'fields'        => 'ids',
            ));
        }

        public function archive( $atts, $content )
        {
            global $wp_rewrite, $post;

            $this->load_styles_and_script();

            $atts = shortcode_atts( array(
                'per_page'          => 30,
                'orderby'           => 'date',
                'column_count'      => 5,
                'column_gap'        => '5px',
                'order'             => 'DESC', 
                'thumb_size'        => 'thumbnail',

            ), $atts );

            extract($atts);

            $html = array();
            $html[] = '<div class="mg-meme-archive-page">';
            $html[] =   '<h1>' . ( $post->post_title ) . '</h1>';

            $paged = ( get_query_var( 'paged' ) ) ? get_query_var( 'paged' ) : 1;
            $args = array(
                'post_type'         => 'generated_memes',
                'order'             => $order,
                'posts_per_page'    => $per_page,
                'paged'             => $paged,
                'orderby'           => $orderby
            );

            
            $wp_query = new WP_Query( $args );

            // use the query for paging
            $wp_query->query_vars[ 'paged' ] > 1 ? $current = $wp_query->query_vars[ 'paged' ] : $current = 1;

            $pagination = array(
                'base' => @add_query_arg( 'paged', '%#%' ),
                'showall' => false,
                'end_size' => 4,
                'mid_size' => 4,
                'total' => $wp_query->max_num_pages,
                'current' => $paged,
                'type' => 'plain'
            );

            // build the paging links
            if ( $wp_rewrite->using_permalinks() )
                $pagination[ 'base' ] = user_trailingslashit( trailingslashit( remove_query_arg( 's', get_pagenum_link( 1 ) ) ) . 'page/%#%/', 'paged' );

            // more paging links
            if ( !empty( $wp_query->query_vars[ 's' ] ) )
                $pagination[ 'add_args' ] = array( 's' => get_query_var( 's' ) );

            if($wp_query->have_posts() ) {
                
                $html[] = '<div class="mg-memes-lists">';
                while ( $wp_query->have_posts() ) {
                    $wp_query->the_post();
                    $item_id = get_the_ID();

                    $thumb = get_post_meta( $item_id, "_mg_attach_id", true );
                    $template = get_post_meta( $item_id, "_mg_template", true );
                    $user_id = (int)get_post_meta( $item_id, "_mg_user_id", true );

                    $user_info = get_userdata( $user_id );
                    $user_nicename = isset($user_info->data->user_nicename) ? $user_info->data->user_nicename : 'anonymous';

                    if( $thumb ){
                        $full_img = wp_get_attachment_url( $thumb );
                        $thumb = wp_get_attachment_image( $thumb, $thumb_size );
                        
                        $html[] =  '<div class="item" data-url="' . ( $full_img ) . '" '; 
                        $html[] =  "data-template='" . ( $item_id ) . "'>"; 
                        $html[] =   '<a href="' . ( get_permalink($item_id ) ) . '">'; 
                        $html[] =       $thumb; 
                        $html[] =   '</a>'; 
                        $html[] =   '<h4>' . ( get_the_title( $template ) ) . '</h4>'; 

                        if( $user_id == 0 ){
                            $html[] =   '<span class="mg-author-anonymous">' . ( $user_nicename ) . '</span>'; 
                        }else{
                            $html[] =   '<span class="mg-author">' . ( $user_nicename ) . '</span>'; 
                        }
                        $html[] =  '</div>'; 
                    }
                }
                $html[] =  '</div>'; 

                $html[] = '<style>';
                $html[] = 'body .mg-meme-archive-page .mg-memes-lists .item {';
                $html[] =   'width: calc((100% / ' . ( (int) $column_count ) . '));';
                $html[] =   'padding-right: ' . ( $column_gap ) . ';';
                $html[] = '}';
                $html[] = '</style>';

                wp_reset_postdata();
            }


            $html[] = '<div class="mg-navigation">' . paginate_links($pagination) . '</div>';

            $html[] = '</div>';

            return implode( "\n", $html );
        }

        public function related_memes( $order_type )
        {
            $html = array();
            $html[] = '<div class="mg-meme-related">';
            $html[] =   '<h2>Generated memes</h2>';

            $args = array(
                'post_type'=> 'generated_memes',
                'order'    => 'ASC',
                'posts_per_page' => 10,
                'orderby'        => 'rand'
            );              

            $memes = new WP_Query( $args );

            if($memes->have_posts() ) {
                $html[] =       '<div class="mg-carousel-holder">';
                
                $html[] =           '<div id="mg-carousel-related">';
                while ( $memes->have_posts() ) {
                    $memes->the_post();
                    $item_id = get_the_ID();

                    $thumb = get_post_meta( $item_id, "_mg_attach_id", true );
                    if( $thumb ){
                        $full_img = wp_get_attachment_url( $thumb );
                        $thumb = wp_get_attachment_image( $thumb, array( 150, 150 ) );

                        //$html[] =  '<div class="item" data-url="' . ( $full_img ) . '" data-options="' . ( json_encode($options) ) . '">' . ( $thumb ) . '</div>'; 
                        $html[] =  '<div class="item" data-url="' . ( $full_img ) . '" '; 
                        $html[] =  "data-template='" . ( $item_id ) . "'>"; 
                        $html[] =   '<a href="' . ( get_permalink($item_id ) ) . '">'; 
                        $html[] =       $thumb; 
                        $html[] =   '</a>'; 
                        $html[] =  '</div>'; 
                    }
                }

                wp_reset_postdata();
                $html[] =           '</div>';
                $html[] =       '</div>';
            }


            $html[] = '</div>';

            return implode( "\n", $html );
        }

        public function generator_page( $atts )
        {
            $atts = shortcode_atts( array(
            ), $atts );

            $this->load_styles_and_script();

            $html = array();

            $post_id = isset($_REQUEST['template']) ? (int)$_REQUEST['template'] : 0;
            if( $post_id == 0 ){
                $templates = $this->get_meme_templates();
                $post_id = isset($templates[0]) ? $templates[0] : 0;
            }
            

            //$post_id = 0;
            if( $post_id > 0 ){
                $post = get_post( $post_id );

                $main_image = get_post_meta( $post_id, "_mg_main_image", true );
                $main_image = wp_get_attachment_url( $main_image );

                if( count($this->cfg['fonts']) > 0 ){
                    foreach ($this->cfg['fonts'] as $font) {
                        if( strlen($font) > 5 ){
                             $html[] =   '<div class="mg-font-render" style="font-family: ' . ( $font ) . '">just render</div>';
                        }
                    }
                }
               

                $html[] = '<div class="mg-container meme-generator-wrapper mg-tpl-' . ( MG()->settings['template'] ) . '">';
                
                $html[] =   '<div class="mg-lightbox">';
                $html[] =       '<div class="mg-lightbox-content">';
                $html[] =           '<a class="mg-close-lightbox">X</a>';
                $html[] =           '<h2>All memes templates <a href="#"></a></h2>';
                $html[] =           '<div class="mg-lightbox-wrapper">';
                $html[] =               '<div class="mg-lightbox-memes-lists"></div>';
                $html[] =               '<div class="mg-lightbox-memes-navs"></div>';
                $html[] =           '</div>';
                $html[] =           '<div class="mg-lightbox-loading">loading ...</div>';
                $html[] =       '</div>';
                $html[] =   '</div>';
                $html[] =   '<div class="mg-left">';
                $html[] =       '<div class="mg-controls">';
                $html[] =           '<button class="mg-b-invert" id="mg-upload-own-image">Upload your own image</button>';
                $html[] =           '<input type="file" id="meme-generator-file-input" />';
                $html[] =       '</div>';
                $html[] =       '<div class="mg-generator">';
                $html[] =           '<div id="meme-generator-main-image">';
                $html[] =               '<canvas width="0" height="0"></canvas>';
                $html[] =           '</div>';
                $html[] =       '</div>';
                $html[] =       '<div class="mg-meme-name">';
                if ( isset($post->post_title) ) {
                    $html[] =           '<h4>' . ( $post->post_title ) . '</h4>';
                }
                $html[] =       '</div>';

                $args = array(
                    'post_type'=> 'memes_templates',
                    'order'    => 'ASC',
                    'orderby'   => 'meta_value_num',
                    'meta_key'  => '_mg_used',
                    'posts_per_page' => 15
                );              

                $memes = new WP_Query( $args ); 
                if($memes->have_posts() ) {
                    $html[] =       '<div class="mg-carousel-holder">';
                    $html[] =           '<h3>Other Templates for Memes </h3>';

                    $html[] =           '<div id="mg-carousel">';
                    while ( $memes->have_posts() ) {
                        $memes->the_post();
                        $item_id = get_the_ID();
                        $item_title = get_the_title();

                        $dboptions = get_post_meta( $item_id, "_mg_options", true );
      
                        $thumb = get_post_meta( $item_id, "_mg_main_image", true );
                        if( $thumb ){
                            $full_img = wp_get_attachment_url( $thumb );
                            $thumb = wp_get_attachment_image( $thumb, array( 75, 75 ) );

                            if( $dboptions ){
                                $options = array();

                                foreach ($dboptions as $value) {
                                    if( !isset($value['text-position']) ){
                                        $value['text-position'] = 'top';
                                    }
                                     $options[] = array(
                                        'pos' => array(
                                            'x' => $value['x'],
                                            'y' => $value['y'],
                                            'width' => $value['width'],
                                            'height' => $value['height']
                                        ),
                                        'placeholder' => $value['placeholder'],
                                        'text-position' => $value['text-position'],
                                        'colors' => array(
                                            'font' => '#ffffff',
                                            'shadow' => '#000000'
                                        )
                                    );
                                }
                            }
                             
                            //$html[] =  '<div class="item" data-url="' . ( $full_img ) . '" data-options="' . ( json_encode($options) ) . '">' . ( $thumb ) . '</div>'; 
                            $html[] =  '<div class="item" data-title="' . ( $item_title ) . '" data-url="' . ( $full_img ) . '" '; 
                            $html[] =  "data-options='" . ( json_encode($options) ) . "'"; 
                            $html[] =  "data-template='" . ( $item_id ) . "'"; 
                            $html[] =  '>' . ( $thumb ) . '</div>'; 
                        }
                    }

                    wp_reset_postdata();
                    $html[] =           '</div>';
                    $html[] =       '</div>';
                }
                $html[] =   '</div>';

                $html[] =   '<div class="mg-right">';
                $html[] =       '<div class="mg-meme-options">';
                $html[] =           '<div class="mg-text-boxes"></div>';
                $html[] =               '<div class="mg-text-options-box">';
                $html[] =                   '<button id="mg-options">';
                $html[] =                       '<i class="fa fa-caret-up"></i>';
                $html[] =                       '<span data-switch="show options">hide options</span>';
                $html[] =                   '</button>';
                $html[] =                   '<div class="mg-text-options">';
                $html[] =                       '<button id="mg-add-another-box">Add another textbox</button><br>';
                $html[] =                       '<label>Font</label>';
                $html[] =                       '<select class="mg-text-font-family">';
                $html[] =                           '<option selected value="Impact">Impact</option>';
                $html[] =                           '<option value="Arial">Arial</option>';
                $html[] =                           '<option value="Comic Sans">Comic Sans</option>';
                $html[] =                       '</select><br>';
                $html[] =                       '<label>Max Font Size <input type="number" name="font-size" value="50" min="1" max="999"> px</label><br>';
                $html[] =                       '<label><input type="checkbox" name="font-shadow" checked> Font Shadow</label><br>';
                $html[] =                       '<label><input type="checkbox" name="all-caps"> USE ALL CAPS</label><br>';
                //$html[] =                       '<label><input type="checkbox" name="drag-drop" checked> Enable drag/drop &amp; resize</label><br>';
                //$html[] =                       '<label><input type="checkbox" name="no-resize"> Use resolution of original template image, do not resize.<br> Potentially higher quality, but larger filesize.</label><br>';
                $html[] =                   '</div>';
                $html[] =                   '<div class="mg-generate-meme">';
                $html[] =                       '<button id="mg-action-generate-meme">Generate Meme</button>';

                $html[] =                       '<a href="#" id="mg-reset-options" class="mg-b-invert">Reset</a>';
                $html[] =                   '</div>';
                $html[] =               '</div><!-- ./mg-text-options-box -->';
                $html[] =           '</div>';
                $html[] =       '</div>';

                $html[] = $this->related_memes( 'random' );

                $html[] = '</div><!-- ./mg-container -->';
                $html[] =  '<div id="mg-generate-test-img">dsadsdsa</div>'; 


                $html[] = '<script type="text/javascript">';
                $html[] =   'jQuery(document).ready(function(){';
                $html[] =       'MemeGenerator.set_ajaxurl( "' . ( admin_url( 'admin-ajax.php' ) ) . '" );';
                $html[] =       'MemeGenerator.set_template( "' . ( $post_id ) . '" );';
                $html[] =       'MemeGenerator.load_image( "' . ( $main_image ) . '" );';
                $html[] =       'MemeGenerator.set_watermark( "' . ( MG()->settings['copyright_text'] ) . '" );';
                $html[] =       "MemeGenerator.set_watermark_style( '" . ( isset(MG()->settings['template_typography']['watermark']) ? json_encode( MG()->settings['template_typography']['watermark'] ) : '{"family":"475","size":"12","weight":"normal","color":"#fff"}' ) . "' );";

                $options = get_post_meta( $post_id, "_mg_options", true );
                if( $options && count($options) > 0 ){
                    foreach ($options as $key => $value) { 

                        if( !isset($value['text-position']) ){
                            $value['text-position'] = 'top';
                        }
                        $html[] =       'MemeGenerator.set_caption({';
                        $html[] =           "'pos'           : {";
                        $html[] =               "'x': '" . ( $value['x'] ) . "',";
                        $html[] =               "'y': '" . ( $value['y'] ) . "',";
                        $html[] =               "'width': '" . ( $value['width'] ) . "',";
                        $html[] =               "'height': '" . ( $value['height'] ) . "'";
                        $html[] =           "},";
                        $html[] =           "'placeholder'   : '" . ( $value['placeholder'] ) . "',";
                        $html[] =           "'text-position'   : '" . ( $value['text-position'] ) . "',";
                        $html[] =           "'colors'        : {";
                        $html[] =               "'font'  : '#ffffff',";
                        $html[] =               "'shadow': '#000000'";
                        $html[] =           "}";
                        $html[] =       "});";
                    }
                }

                $html[] =   '});';
                $html[] = '</script>';
            }else{
                $html[] =   '<div class="mg-container"><div class="mg-no-templates-error">You need to add some meme templates first</div></div>';
            }

            return implode( "\n", $html );
        }

        public function template_loader()
        {
            global $wp_query;

            $file = 'archive';
            // in game page 
            if( isset($wp_query->tax_query->queries) ){
                $queried_object = get_queried_object();

                if( isset($queried_object->taxonomy) && $queried_object->taxonomy == 'amzproductcategs' ){
                    $term_taxonomy_id = isset($queried_object->term_taxonomy_id) ? (int)$queried_object->term_taxonomy_id : 0;
                    $GLOBALS['term_taxonomy_id'] = $term_taxonomy_id;
                    if( $term_taxonomy_id > 0 ){
                        $file = 'game';
                    }
                }
            }

            if( file_exists(MG()->path( 'TEMPLATES_DIR', $file  . '.php' )) ){

                require_once MG()->path( 'TEMPLATES_DIR', 'header.php' );
                require_once MG()->path( 'TEMPLATES_DIR', $file  . '.php' );

                die;
            }
        }

        public function make_request()
        {
            $vaild_actions = array( 'save_meme', 'frontend_style' );
            $action = isset($_REQUEST['sub_action']) ? $_REQUEST['sub_action'] : '';
 
            if( in_array( $action, $vaild_actions) ){

                if( $action == 'save_meme' ){
                    $image = isset($_REQUEST['image']) ? $_REQUEST['image'] : '';

                    if( trim($image) != "" ){

                        $image = str_replace("data:image/png;base64,", "", $image);
                        $image = base64_decode( $image );
                        $user = wp_get_current_user();


                        $my_post = array(
                          'post_title'    => wp_strip_all_tags( "Untitled" ),
                          'post_status'   => 'publish',
                          'post_author'   => 1,
                          'post_type'     => 'generated_memes'
                        );
                         
                        // Insert the post into the database
                        $pid = wp_insert_post( $my_post );
 
                        $upload_dir = wp_upload_dir();
                        $filename = MG()->alias . '_' . uniqid() . '.png';
                        $image_path = $upload_dir["path"] . '/' . $filename;
   
                        //write image if the wp method fails
                        $has_wrote = MG()->wp_filesystem->put_contents(
                            $image_path, $image, FS_CHMOD_FILE
                        );

                        if( !$has_wrote ){
                            file_put_contents( $image_path, $image );
                        }

                        // Add image in the media library - Step 3
                        $wp_filetype = wp_check_filetype( basename( $image_path ), null );
                        $attachment = array(
                            // 'guid'           => $image_url,
                            'post_mime_type' => $wp_filetype['type'],
                            'post_title'     => preg_replace( '/\.[^.]+$/', '', basename( $image_path ) ),
                            'post_content'   => '',
                            'post_status'    => 'inherit'
                        );
     
                        $attach_id = wp_insert_attachment( $attachment, $image_path, $pid  ); 
                        require_once( ABSPATH . 'wp-admin/includes/image.php' );
                        $attach_data = wp_generate_attachment_metadata( $attach_id, $image_path );
                        wp_update_attachment_metadata( $attach_id, $attach_data );

                        update_post_meta( $pid, "_mg_attach_id", $attach_id );

                        update_post_meta( $pid, "_mg_user_id", (int)$user->ID );
                        update_post_meta( $pid, "_mg_template", (int)$_REQUEST['template'] );

                        $used = (int) get_post_meta( (int)$_REQUEST['template'], '_mg_used', true );
                        update_post_meta( (int)$_REQUEST['template'], "_mg_used", ($used + 1) );

                        // Update the post into the database
                        wp_update_post( array(
                            'ID'           => $pid,
                            'post_content' => '[meme_generator_post attach_id="' . ( $attach_id ) . '"]'
                        ) );

                        die( json_encode(array(
                            'status' => 'valid',
                            'redirect_url' => get_permalink( $pid )
                        )));
                    }
                }
                elseif( $action == 'frontend_style' ){

                    $css = MG()->wp_filesystem->get_contents( MG()->path( 'TEMPLATES_DIR', 'css/style.css' ) );
                    if( $css == false ){
                        $css = file_get_contents( MG()->path( 'TEMPLATES_DIR', 'css/style.css' ) );
                    }
                    
                    $css = $this->fix_buffer_hover( $css );

                    header("Content-type: text/css");

                    // Remove space after colons
                    $css = str_replace(': ', ':', $css);
                     
                    // Remove whitespace
                    $css = str_replace(array("\r\n", "\r", "\n", "\t", '    ', '    '), '', $css);
                     
                    // Enable GZip encoding.
                    if ( ! ini_get('zlib.output_compression') || 'ob_gzhandler' != ini_get('output_handler') ) ob_start();
                    else ob_start("ob_gzhandler");

                    echo $css;

                    die;
                }
            }
            
            die("Invalid Action!");
        }

        public function fix_buffer_hover( $buffer='' ) 
        {
            $found = substr_count( $buffer, ':hover' );
            $buffer = $this->_fix_buffer_hover_acolade($buffer);
            $found2 = substr_count( $buffer, 'is_hover' );
            return $buffer;
        }
        
        public function _fix_buffer_hover_acolade( $val='' ) 
        {
            if ( ! strstr( $val, '{' ) ) {
                return $val;
            }

            $_ = explode('{', $val);
            foreach ($_ as $kk => $vv) { // foreach 1
                $_2 = explode('}', $vv);
                foreach ($_2 as $kk2 => $vv2) { // foreach 2
                    $_3 = explode(',', $vv2);
                    foreach ($_3 as $kk3 => $vv3) { // foreach 3
                        $vv3a = str_replace(':hover', '.is_hover', $vv3);
                        if ( $vv3 != $vv3a ) {
                            $vv3a = $vv3 . ', ' . $vv3a;
                        }
                        $_3["$kk3"] = $vv3a;
                    } // end foreach 3
                    $_2["$kk2"] = implode(',', $_3);
                } // end foreach 2
                $_["$kk"] = implode('}', $_2);
            } // end foreach 1

            $val = implode('{', $_);
            return $val;
        }

        public function get_all_generated_memes()
        {
            return get_posts( array(
                'numberposts'   => -1,
                'post_type'     => 'generated_memes',
                'fields'        => 'ids',
            ));
        }

        public function get_meme_navs( $current_id=0 )
        {
            $memes = $this->get_all_generated_memes();
            $navs = array(
                'prev' => 0,
                'next' => 0,
                'random' => 0,
                'current' => 0,
            );
            if( $memes ){
                if( in_array( $current_id, $memes)) {
                    $current_key = array_search( $current_id, $memes);
                    $navs['current'] = $memes[$current_key];
                    
                    if( isset($memes[$current_key-1]) ){
                        $navs['prev'] = $memes[$current_key-1];
                    }
                    if( isset($memes[$current_key+1]) ){
                        $navs['next'] = $memes[$current_key+1];
                    }

                    unset($memes[$current_key]);
                    if( count($memes) > 0 ){
                        shuffle( $memes );
                        $navs['random'] = $memes[0];
                    }
                }
            }

            return $navs;
        }

        public function get_page( $alias='', $extra='' )
        {
            $post_id = 0;
            $post_url = '';

            if( $alias == 'meme-generator' )
            {
                if( isset(MG()->settings['generator_page']) ){

                    $post_id = (int)MG()->settings['generator_page'];
                }
            }

            if( $alias == 'meme_archive_page' )
            {
                if( isset(MG()->settings['meme_archive_page']) ){

                    $post_id = (int)MG()->settings['meme_archive_page'];
                }
            }

            $post_url = get_permalink( $post_id );

            if( trim($extra) != "" ){
                $post_url .= $extra;
            }

            return array(
                'post_id' => $post_id,
                'post_url' => $post_url
            );
        }

        public function suppress_title( $title, $id = null ) 
        {
            return "";
        }

        public function post_page( $atts )
        {
            global $post;

            $atts = shortcode_atts( array(
                'attach_id' => 0
            ), $atts, 'meme_generator_post' );

            $this->load_styles_and_script();
            $attach_id = isset($atts['attach_id']) ? (int) $atts['attach_id'] : 0;

            $archive_page_link = $this->get_page( 'meme_archive_page' );

            $html = array();
            if( $attach_id > 0 ){


                $attachment = wp_get_attachment_image( $attach_id, array('800', '800'), "", array( "class" => "img-responsive" ) );
                $attachment_url = wp_get_attachment_url( $attach_id, array('800', '800') );

                if( $attachment != "" ){
                    $html[] = '<div class="mg-meme-detail">';
                    $html[] =   '<div class="mg-meme-detail-container">';
                    $html[] =       '<div class="mg-meme-image">';
                    $html[] =           $attachment;
                    $html[] =       '</div>';

                    $html[] =       '<div class="mg-meme-controls">';
                    if( is_post_type_archive( 'generated_memes')  !== true && $post->post_type != 'generated_memes' ){
                        $navs = $this->get_meme_navs( $post->ID );
                        

                        if( $navs['prev'] > 0 ){
                            $html[] =       '<a href="' . ( get_permalink( $navs['prev'] ) ) . '"><i class="fa fa-long-arrow-left" aria-hidden="true"></i> previous meme</a>';
                        }
                        if( $navs['next'] > 0 ){
                            $html[] =       '<a href="' . ( get_permalink( $navs['next'] ) ) . '"><i class="fa fa-long-arrow-right" aria-hidden="true"></i> next meme</a>';
                        }
                        if( $navs['random'] > 0 ){
                            $html[] =       '<a href="' . ( get_permalink( $navs['random'] ) ) . '"><i class="fa fa-random" aria-hidden="true"></i> random meme</a>';
                        }
                    }   

                    $template = (int) get_post_meta( $post->ID, '_mg_template', true );
                    $recaption_page = $this->get_page( 'meme-generator', '?template=' . $template );

                    if( is_post_type_archive( 'generated_memes')  === true || $post->post_type == 'generated_memes' ){
                        $html[] =       '<a style="margin-top: 0px;" href="' . ( get_permalink( $post->ID ) ) . '" class="mg-recaption">View meme</a>';
                    }
                     
                    $html[] =           '<a href="' . ( $recaption_page['post_url'] ) . '" class="mg-recaption">Re-Caption this meme</a>';
                    $html[] =           '<a href="#" class="mg-recaption" id="mg-show-embed-code">Show embed codes</a>';

                    if( is_singular( 'generated_memes' ) === true || $post->post_type != 'generated_memes' ){
                        $html[] = '<a href="' . ( $archive_page_link['post_url'] ) . '" class="mg-recaption">Generated Memes Archive</a>';
                    }

                    $html[] =           '<div class="mg-embed-codes-wrapper">';
                    $html[] =               '<div class="mg-embed-codes">';
                    $html[] =                   '<label><span>Image Link:</span> <input onClick="this.setSelectionRange(0, this.value.length)" type="text" value="' . ( $attachment_url ) . '" /></label>';
                    $html[] =                   '<label><span>Image HTML:</span> <input onClick="this.setSelectionRange(0, this.value.length)" type="text" value=\'<a href="' . ( get_the_permalink( $post->ID ) ) . '"><img src="' . ( $attachment_url ) . '" title="made at ' . ( get_bloginfo( 'name' ) ) . '"/></a>\' /></label>';
                    $html[] =               '</div>';
                    $html[] =           '</div>';

                    $user = (int) get_post_meta( $post->ID, '_mg_user_id', true );
                    $print_user = 'anonymous';
                    if( $user > 0 ){
                        $author_obj = get_user_by('id', $user);
                        $print_user = $author_obj->data->user_nicename;
                        $html[] =     '<p>Made by <span class="mg-author-url">' . ( $print_user ) . '</span> ' . ( human_time_diff( get_the_time('U'), current_time('timestamp') ) . ' ago' ) . '.</p>';
                    }else{
                        $html[] =     '<p>Made by <span style="color: #9299a0">' . ( $print_user ) . '</span> ' . ( human_time_diff( get_the_time('U'), current_time('timestamp') ) . ' ago' ) . '.</p>';
                    }


                    $html[] =       '</div>';


                    $html[] =   '</div>';

                    if( is_post_type_archive( 'generated_memes')  !== true && ( is_author() === false && $post->post_type == 'generated_memes' ) ){
                        $html[] =   '<div class="mg-social-share">';
                        $html[] =       '<span class="mg-share"><i class="fa fa-share" aria-hidden="true"></i> <img src="' . ( MG()->templatesUrl('/img/ajax-loader.gif') )  . '" data-id="' . ( $post->ID ) . '" id="mg-total-shares" /></span>';
                        $html[] =       '<a target="_blank" href="https://www.facebook.com/sharer/sharer.php?u=' . ( urlencode(get_permalink( $post->ID) ) ) . '" class="mg-facebook"><i class="fa fa-facebook" aria-hidden="true"></i> Share on Facebook</a>';;
                        $html[] =       '<a target="_blank" href="https://plus.google.com/share?url=' . ( urlencode(get_permalink( $post->ID) ) ) . '" class="mg-google-plus"><i class="fa fa-google-plus" aria-hidden="true"></i> Share on Google</a>';
                        $html[] =       '<a target="_blank" href="https://www.pinterest.com/join/?next=/pin/create/button/%3Furl%3D' . ( urlencode(get_permalink( $post->ID) ) ) . '" class="mg-pinterest"><i class="fa fa-pinterest-p" aria-hidden="true"></i> Share on Pinterest</a>';
                        $html[] =   '</div>';
                        $html[] = '<script type="text/javascript">';
                        $html[] =   'jQuery(document).ready(function(){';
                        $html[] =       'MemeGenerator.set_ajaxurl( "' . ( admin_url( 'admin-ajax.php' ) ) . '" );';
                        $html[] =   '});';
                        $html[] =   '</script>';
                    }

                    if( is_post_type_archive( 'generated_memes')  !== true && ( is_author() === false && $post->post_type == 'generated_memes' ) ){
                        $html[] = $this->related_memes( 'random' );
                    }
                    
                    $html[] = '</div>';

                    $local_view = (int) get_post_meta( $post->ID, '_mg_hits', true );
                    $local_view = $local_view + 1;
                    update_post_meta( $post->ID, '_mg_hits', $local_view ); 

                }else{
                    $html[] = 'invalid attachment'; 
                }
            }else{
                $html[] = 'invalid attachment';
            }

            echo implode( "\n", $html );
        }

        public function list_page( $atts )
        {
            $atts = shortcode_atts( array(
            ), $atts, 'meme_generator_list_page' );

            $html = array();
            $html[] = 'some list pages ...';

            echo implode( "\n", $html );
        }

        public function login_from( $atts )
        {
            $mg_id = 180;
            if (is_user_logged_in()) {               
                echo '<div class="logout"> <p>Hello!<div class="logout_user"> You are logged in and can proceed to the <a href="' . ( get_permalink( $mg_id ) ) . '">' . ( get_the_title($mg_id) ) . '</a>.</div></p><br /><p><a id="wp-submit" class="logout" href="', wp_logout_url( get_permalink( $mg_id ) ), '" title="Logout">Logout</a></p></div>';
            } else { 
                $args = array(
                    'echo'           => true,
                    'redirect'       => get_permalink( $mg_id ),
                    'label_log_in'   => __( 'Log in' ),
                    'form_id'        => 'meme-login',
                    'label_username' => __( 'Username' ),
                    'label_password' => __( 'Password' ),
                    'label_remember' => __( 'Remember Me' ),
                    'id_username'    => 'user_login',
                    'id_password'    => 'user_pass',
                    'id_submit'      => 'wp-submit',
                    'remember'       => true,
                    'value_username' => NULL,
                    'value_remember' => true
                );
                wp_login_form($args); 
            }
        }
    }
}