/*
Document   :  MGTemplates
Author     :  AA-Team http://themeforest.net/user/AA-Team
*/
// Initialization and events code for the app
MGTemplates = (function ($) {
    "use strict";

    // public
    var debug_level = 0;
    var maincontainer = null;
    var template_text = null;
    var options_text = null;
    var option_id = 0;
    var file_frame, image_data, mainimage;

	// init function, autoload
	(function init() {
		// load the triggers
		$(document).ready(function(){
			maincontainer = $("#MG_meme_extra");
			options_text = maincontainer.find(".MG_options_text");
			mainimage = maincontainer.find(".MG_configurator_main_image_wrapper");
			template_text = maincontainer.find("#MG_text-tpl-html").html();
			triggers();
			populate_options();
		});
	})();
	
	function populate_options()
	{
		$.each( options_text.data("options"), function(key, value){
			var _template_text = template_text.replace( /option_id/g, key );
			var uniqid = "meme-generator-settings-" + key;

			_template_text = _template_text.replace( "option_unique_id", uniqid );

			options_text.append( _template_text );

			var new_option = options_text.find("." + uniqid );
			new_option.find(".MG-option-width").val( value.width );
			new_option.find(".MG-option-height").val( value.height );
			new_option.find(".MG-option-x").val( value.x );
			new_option.find(".MG-option-y").val( value.y );
			new_option.find(".MG-option-placeholder").val( value.placeholder );


			var text_box = $("<div />");
			text_box.addClass('MG_resizeable_box');
			text_box.data( 'uniqid', uniqid );

			var handles_pos = [];
			handles_pos.push('nw');
			handles_pos.push('ne');
			handles_pos.push('sw');
			handles_pos.push('se');
			handles_pos.push('n');
			handles_pos.push('s');
			handles_pos.push('e');
			handles_pos.push('w');

			$.each(handles_pos, function(key, val){
				text_box.append( '<div class="ui-resizable-handle ui-resizable-' + ( val ) + '" />' );
			});

			var placeholder = $("<span />");
			placeholder.addClass('MG_resizeable_placeholder');
			placeholder.text( value.placeholder );
			text_box.append( placeholder );
			mainimage.append( text_box );

			text_box.css({
				'top': value.y + "%",
				'left': value.x + "%",
				'width': value.width + "%",
				'height': value.height + "%"
			});

			make_resizable( text_box );
		});

		option_id = options_text.find("li").size();
	}

	function new_textbox()
	{
		var uniqid = get_uniqid();
		
		option_id++;
		var _template_text = template_text.replace( /option_id/g, option_id );

		_template_text = _template_text.replace( "option_unique_id", uniqid );

		options_text.append( _template_text );

		var text_box = $("<div />");
		text_box.addClass('MG_resizeable_box');
		text_box.data( 'uniqid', uniqid );

		var handles_pos = [];
		handles_pos.push('nw');
		handles_pos.push('ne');
		handles_pos.push('sw');
		handles_pos.push('se');
		handles_pos.push('n');
		handles_pos.push('s');
		handles_pos.push('e');
		handles_pos.push('w');

		$.each(handles_pos, function(key, val){
			text_box.append( '<div class="ui-resizable-handle ui-resizable-' + ( val ) + '" />' );
		});

		var placeholder = $("<span />");
		placeholder.addClass('MG_resizeable_placeholder');
		placeholder.text( "Text here" );
		text_box.append( placeholder );
		mainimage.append( text_box );

		make_resizable( text_box );
	}

	function percentage( x, y )
	{
		return Number(x * 100 / y).toFixed(2);
	}

	function make_resizable( box )
	{
		box.draggable({
			containment: mainimage,
			stop: function( event, ui ) {
				var option_box = options_text.find( "." + $(ui.helper).data( 'uniqid' ) );

				option_box.find(".MG-option-y").val( percentage( ui.position.top, mainimage.height() ));
				option_box.find(".MG-option-x").val( percentage( ui.position.left, mainimage.width() ));
			}
	    });

		box.resizable({
			containment: mainimage,
		    handles: {
		        'nw': '.ui-resizable-nw',
		        'ne': '.ui-resizable-ne',
		        'sw': '.ui-resizable-sw',
		        'se': '.ui-resizable-se',
		        'n': '.ui-resizable-n',
		        'e': '.ui-resizable-e',
		        's': '.ui-resizable-s',
		        'w': '.ui-resizable-w'
		    },
		    stop: function( event, ui ) {
		    	var option_box = options_text.find( "." + $(ui.helper).data( 'uniqid' ) );

				option_box.find(".MG-option-width").val( percentage( ui.size.width, mainimage.width() ));
				option_box.find(".MG-option-height").val( percentage( ui.size.height, mainimage.height() ));
		    }
		});
	}

	function get_uniqid()
	{
		var n=Math.floor(Math.random()*11);
		var k = Math.floor(Math.random()* 1000000000);
		return $.trim( 'meme-generator-settings-' + $.trim( k ));
	}

	function triggers()
	{
		maincontainer.on('click', ".MG_add_new_textbox", function(e){
			e.preventDefault();
			new_textbox();
		});

		maincontainer.on('click', ".MG_delete_text", function(e){
			e.preventDefault();
			
			$(this).parents('li').eq(0).remove();
		});

		maincontainer.on('click', ".MG_upload_image_btn", function(){

			file_frame = wp.media.frames.file_frame = wp.media({
		        title: 'Insert a media',
                library: {type: 'image'},
                multiple: false,
                button: {text: 'Insert meme image'}
		    });

		    file_frame.open();

		    file_frame.on( 'select', function() {
		    	// Get media attachment details from the frame state
      			var attachment = file_frame.state().get('selection').first().toJSON();

      			var img = $("<img />"),
				input = $("<input type='hidden' name='MG[main_image]' />");

				img.attr( 'src', attachment.url );
				input.val( attachment.id );

				mainimage.html( img ).append( input );
			});

			return false;
		});
	}

	// external usage
	return {
	}
})(jQuery);