<?php 
/**
 * memes post type class
 * =====================
 *
 * @author		AA-Team
 * @version		1.0
 * @access 		public
 * @return 		void
 */  
!defined('ABSPATH') and exit;
if (class_exists('MGGeneratedMemes') != true) {
	class MGGeneratedMemes
	{
		/*
		* Some required plugin information
		*/
		const VERSION = '1.0';

		/*
		* Store some helpers config
		*/
		public $the_plugin = null;

		public $aaAmazonWS = null;
		public $_settings = array();

		static protected $_instance;

		public function __construct()
		{
			$this->the_plugin = $GLOBALS['MG'];
			
			add_action( "init", array( $this, 'init_post_type' ) );
  
			/* use save_post action to handle data entered */
			//add_action( 'save_post', array( $this, 'meta_box_save_postdata' ) );
			add_action( 'admin_head', array( $this, 'load_extra_styles_admin' ) ); 
			
			if( isset($_GET['post_type']) && $_GET['post_type'] == 'generated_memes') {
				add_action('admin_head', array( $this, 'extra_css') );
			}
			
			// define the ajax requests
			add_action( 'wp_ajax_MG_pt_meme_ajax_requests', array( $this, 'ajax_requests' ) );
			add_action( 'wp_ajax_nopriv_MG_pt_meme_ajax_requests', array( $this, 'ajax_requests' ) );
		}
		
		public function load_extra_styles_admin() 
		{
			$screen = get_current_screen();
			if(
				isset( $screen->post_type ) && 
				trim( $screen->post_type ) == 'generated_memes' 				
			) {
				wp_enqueue_style( 'admin-memes-style', $this->the_plugin->cfg["CURRENT_PT_URL"] . 'assets/style.css' );
				wp_enqueue_style( 'font-awesome', 'https://maxcdn.bootstrapcdn.com/font-awesome/4.4.0/css/font-awesome.min.css' );

				wp_enqueue_script( 'MG_templates_app', $this->the_plugin->cfg["CURRENT_PT_URL"] . 'assets/app.class.js', array(), '1.0.0', true );

				wp_enqueue_script('jquery-ui-core'); 
				wp_enqueue_script('jquery-ui-resizable'); 
				wp_enqueue_script('jquery-ui-draggable');   
			}	
		}
		
		/**
		* Singleton pattern
		*
		* @return MGGeneratedMemes Singleton instance
		*/
		static public function getInstance()
		{
			if (!self::$_instance) {
				self::$_instance = new self;
			}

			return self::$_instance;
		}

		public function init_post_type() 
		{  
			// get label
			$labels = array(
				'name' 					=> __('Generated Memes', 'MG'),
				'singular_name' 		=> __('meme', 'MG'),
				'add_new' 				=> __('Add new meme', 'MG'),
				'add_new_item' 			=> __('Add new meme', 'MG'),
				'edit_item'			 	=> __('Edit meme', 'MG'),
				'new_item' 				=> __('New meme', 'MG'),
				'view_item' 			=> __('View meme', 'MG'),
				'search_items' 			=> __('Search memes', 'MG'),
				'not_found' 			=> __('No memes found', 'MG'),
				'not_found_in_trash' 	=> __('No memes in trash', 'MG')
			);
		   
			// start formationg arguments
			$args = array(
				'labels' => $labels,
				'public' => true,
				'publicly_queryable' => true,
				'show_ui' => true,
				'has_archive' => true,
				'query_var' => true,
				'rewrite' => true,
				'menu_icon' => MG_asset_url( 'images/icon1.png' ),
				'capability_type' => 'post',
				'show_in_menu' => true,
				'has_archive'  => true,
				'supports' => array( 'title', 'editor', 'author', 'comments' )
			);

			register_post_type( 'generated_memes', $args );

			// change the layout of memes list
			add_filter('manage_edit-generated_memes_columns', array( $this, 'memes_edit_columns' ) );
			add_action('manage_posts_custom_column', array( $this, 'memes_posts_columns' ), 10, 2);  
			
			// add taxonomies for this post type
			$this->init_taxonomies();
			
			// add meta boxes to "memes" post type
			add_action('admin_menu', array($this, 'add_to_menu_metabox'));

			add_action( 'save_post', array( $this, 'save_post' ), 10, 3 );
		}

		// Initialize New Taxonomy Labels for the questions custom post type
		public function init_taxonomies() 
		{
			$labels = array(
				'name' 				=> __( 'memes Categories', 'MG' ),
				'singular_name' 	=> __( 'category', 'MG' ),
				'search_items' 		=> __( 'Search category Types', 'MG' ),
				'all_items' 		=> __( 'All category Types', 'MG' ),
				'parent_item' 		=> __( 'Parent category Types', 'MG' ),
				'parent_item_colon' => __( 'Parent category Types:', 'MG' ),
				'edit_item' 		=> __( 'Edit category Types', 'MG' ),
				'update_item' 		=> __( 'Update category Type', 'MG' ),
				'add_new_item' 		=> __( 'Add New category Type', 'MG' ),
				'new_item_name' 	=> __( 'New category Type', 'MG' ),
			);
			
		
			// register taxonomy to hold our memecategs
			register_taxonomy(
				'memecategs',
				array('memes'),
				array(
					'hierarchical' => true,
					'query_var' => true,
					'show_in_nav_menus' => true,
					'labels' => $labels,
					'show_tagcloud' => false
				)
			);
		}

		public function save_post( $post_id, $post, $update )
		{
			// get the screen object to decide if we want to inject our scripts
			$screen = get_current_screen(); 

			if( isset($screen) && $screen->post_type == 'memes_templates' ){

				$mg_array = isset($_POST['MG']) ? $_POST['MG'] : array();
				if( count($mg_array) > 0 ){
					if( isset($mg_array['main_image']) && (int) $mg_array['main_image'] > 0 ){
						update_post_meta( $post_id, '_mg_main_image', $mg_array['main_image'] );
					}
				}

				if( isset($mg_array['option']) && count($mg_array['option']) > 0 ){
					update_post_meta( $post_id, '_mg_options', $mg_array['option'] );
				} 
			}
		}

		public function add_to_menu_metabox()
		{
			// add meta box to "memes_templates" post type
			add_meta_box(
				$this->the_plugin->alias . '_meme_extra', 
				__('Generated Meme', 'MG'), 
				array($this, 'meme_extra_fileds'), 
				'generated_memes', 
				'normal',
				'high'
			);
		}
		
		public function meme_extra_fileds()
		{
			global $post_id;
			
			$attachment_id = get_post_meta( $post_id, "_mg_attach_id", true );
			
			// debug
			if( 0 ) var_dump('<pre>',$extra,'</pre>');
			
			$html = array();
			$html[] = '<div class="MG_pt_meme_generated_extra">';

			$html[] = 	'<div class="MG_view_generated_meme">';

			$attachment = wp_get_attachment_image( $attachment_id, 'full' );

			$html[] = 		$attachment;
			$html[] = 	'</div>';

			$html[] = '</div>';
			
			echo implode( "\n", $html );
		}

		public function memes_edit_columns($columns) 
		{
			$new_columns['cb'] 					= '<input type="checkbox" />';
			//$new_columns['meme_id'] 			= __('ID', 'MG');
			$new_columns['generated_meme_thumbnail'] 	= __('Image', 'MG');
			$new_columns['generated_meme_user'] 		= __('User', 'MG');
			$new_columns['statistics'] 			= __('Statistics', 'MG');
			$new_columns['title'] 				= __('Title', 'MG');
			$new_columns['date'] 				= __('Date', 'MG');
		
			return $new_columns;
		}
		
		public function memes_posts_columns($column_name, $id) 
		{
			global $id;
			switch ($column_name) {
				case 'meme_id':
					echo $id;
					break;
					
				case 'generated_meme_thumbnail':
					$main_image	= get_post_meta( $id, "_mg_attach_id", true );
					$attachment_thumb = wp_get_attachment_image( $main_image, array(75, 75) );
 
					if( $attachment_thumb ){
						echo '<a href="' . ( admin_url( sprintf( 'post.php?post=%s&action=edit', $id ) )) . '">' . $attachment_thumb . '</a>';
					}
					break;

				case 'statistics':
					echo '<div class="mg-share-count-view" id="mg-shares-' . ( $id ) . '"><div id="noTrespassingOuterBarG">
								<div id="noTrespassingFrontBarG" class="noTrespassingAnimationG">
									<div class="noTrespassingBarLineG"></div>
									<div class="noTrespassingBarLineG"></div>
									<div class="noTrespassingBarLineG"></div>
									<div class="noTrespassingBarLineG"></div>
									<div class="noTrespassingBarLineG"></div>
									<div class="noTrespassingBarLineG"></div>
								</div>
							</div><script>jQuery(document).ready(function(){MGTemplates.view_count("' . ( $id ) . '");});</script></div>';
					//MG_Social()->update_shares( 'http://9gag.com/gag/arOKxMd', $id );
					break;
					
				case 'generated_meme_user':
					$user_id	= (int)get_post_meta( $id, "_mg_user_id", true );
					
					if( $user_id > 0 ){
						$user_info = get_userdata( $user_id );
						echo '<a href="#">' . $user_info->data->user_nicename . '</a>';  

					}else{
						echo '<span style="color: #bdc3c7">anonymous</span>';
					}
					break;
					
				default:
					break;
			} // end switch
		}
		
		public function extra_css() 
		{
			echo '<link href="'.$this->the_plugin->cfg["CURRENT_PT_URL"].'admin-memes-style.css" type="text/css" />';
			echo "
				<style type='text/css'>
				th#avatar {width: 50px;}
				th#meme_id {width: 30px;}
				th#generated_meme_user {width: 100px;}
				th#generated_meme_user {width: 100px;}
				th#statistics {width: 200px;}
				th#generated_meme_thumbnail {width: 75px;}
				th#meme_categs {width: 100px;}
				th#meme_date {width: 100px;}
				div.lcf-panel, div.lcf-panel-widget {box-shadow: none;}
				</style>
			";
		}

		public function ajax_requests()
		{
			$action = isset($_REQUEST['sub_action']) ? $_REQUEST['sub_action'] : "";

			if( $action == 'admin_view_statistics' ){
				$post_id = isset($_REQUEST['post_id']) ? (int) $_REQUEST['post_id'] : 0;

				$post_url = get_permalink( $post_id );

				//$post_url = 'http://9gag.com/gag/azrjBdx';
				$shares = MG_Social()->update_shares( $post_url );

				$html = array();
				if( count($shares) > 0 ){
					foreach ($shares as $key => $value) {
						$html[] = '<span class="mg-stats-label mg-stats-' . ( $key ) . '">' . ( $value ) . '</span>';
					}
				}

				$local_view = (int) get_post_meta( $post_id, '_mg_hits', true );
				$html[] = '<span class="mg-stats-label mg-stats-hits">' . ( $local_view ) . '</span>';

				die( json_encode(array(
					'status' => 'valid',
					'data' => $shares,
					'html' => implode("\n", $html)
				)));
			}

			if( $action == 'load_memes' ){

				$totals = MG()->db->get_row( "SELECT count(id) as totals from " . MG()->db->prefix . "posts where 1=1 and post_status='publish' and post_type='memes_templates'" );
				$totals_rows = 0;
				if( isset($totals->totals) && (int)$totals->totals > 0 ){
					$totals_rows = (int)$totals->totals;
				}

				$per_pages = 42;
				$paged = isset($_REQUEST['paged']) ? (int)$_REQUEST['paged'] : 1;
				$args = array(
					'post_type'			=> 'memes_templates',
					'paged'				=> $paged,
					'order'    			=> 'ASC',
					'orderby'   		=> 'meta_value_num',
					'meta_key'  		=> '_mg_used',
					'posts_per_page' 	=> $per_pages
				);

				$html = array();

				$memes = new WP_Query( $args );
				if($memes->have_posts() ) {
					while ( $memes->have_posts() ) {
						$memes->the_post();
						$item_id = get_the_ID();
						$dboptions = get_post_meta( $item_id, "_mg_options", true );
	  
						$thumb = get_post_meta( $item_id, "_mg_main_image", true );
						if( $thumb ){
							$full_img = wp_get_attachment_url( $thumb );
							$name = get_the_title($item_id);
							$thumb = wp_get_attachment_image( $thumb, array( 150, 150 ) );

							if( $dboptions ){
								$options = array();

								foreach ($dboptions as $value) {
									if( !isset($value['text-position']) ){
										$value['text-position'] = 'top';
									}
									 $options[] = array(
										'pos' => array(
											'x' => $value['x'],
											'y' => $value['y'],
											'width' => $value['width'],
											'height' => $value['height']
										),
										'placeholder' => $value['placeholder'],
										'text-position' => $value['text-position'],
										'colors' => array(
											'font' => '#ffffff',
											'shadow' => '#000000'
										)
									);
								}
							}
							 
							$html[] =  '<div data-url="' . ( $full_img ) . '" ';
							$html[] =  "data-options='" . ( json_encode($options) ) . "'";
							$html[] =  "data-template='" . ( $item_id ) . "'";
							$html[] =  "title='" . ( $name ) . "'";
							$html[] =  '>' . ( $thumb ) . '</div>';
						}
					}


					$html_nav = array();
					if( $totals_rows > 0 ){

						$pages = ceil( $totals_rows / $per_pages );
						if( $pages > 0 ){
							for( $cc = 1; $cc <= $pages; $cc++ ){
								$html_nav[] = '<a href="#' . ( $cc ) . '" class="' . ( $cc == $paged ? 'on' : '' ) . '" data-nav="' . ( $cc ) . '">' . ( $cc ) . '</a>';
							}
						}
					}

					die( json_encode( array(
						'status' => 'valid',
						'html' => implode( "\n", $html ),
						'nav' => implode( "\n", $html_nav )
					)));  
				}
			}

		}
	}
}
MGGeneratedMemes::getInstance();