<?php 
/**
 * memes post type class
 * =====================
 *
 * @author		AA-Team
 * @version		1.0
 * @access 		public
 * @return 		void
 */  
!defined('ABSPATH') and exit;
if (class_exists('MGMemes') != true) {
    class MGMemes
    {
        /*
        * Some required plugin information
        */
        const VERSION = '1.0';

        /*
        * Store some helpers config
        */
        public $the_plugin = null;

        public $aaAmazonWS = null;
        public $_settings = array();

		static protected $_instance;

        public function __construct()
        {
        	$this->the_plugin = $GLOBALS['MG'];
			
			add_action( "init", array( $this, 'init_post_type' ) );
  
			add_action( 'admin_head', array( $this, 'load_extra_styles_admin' ) ); 
			
			if( isset($_GET['post_type']) && $_GET['post_type'] == 'memes_templates') {
				add_action('admin_head', array( $this, 'extra_css') );
			}
			
			// define the ajax requests
			add_action( 'wp_ajax_MG_pt_meme_ajax_requests', array( $this, 'ajax_requests' ) );
        }

       
		
		public function load_extra_styles_admin() 
		{
			$screen = get_current_screen();
			if(
				isset( $screen->post_type ) && 
				trim( $screen->post_type ) == 'memes_templates' 				
			) {
				wp_enqueue_style( 'admin-memes-style', $this->the_plugin->cfg["CURRENT_PT_URL"] . 'assets/style.css' );
				wp_enqueue_style( 'font-awesome', 'https://maxcdn.bootstrapcdn.com/font-awesome/4.4.0/css/font-awesome.min.css' );

				wp_enqueue_script( 'MG_templates_app', $this->the_plugin->cfg["CURRENT_PT_URL"] . 'assets/app.class.js', array(), '1.0.0', true );

				wp_enqueue_script('jquery-ui-core'); 
				wp_enqueue_script('jquery-ui-resizable'); 
				wp_enqueue_script('jquery-ui-draggable'); 
				wp_enqueue_media();
			}	
		}
		
		/**
	    * Singleton pattern
	    *
	    * @return MGMemes Singleton instance
	    */
	    static public function getInstance()
	    {
	        if (!self::$_instance) {
	            self::$_instance = new self;
	        }

	        return self::$_instance;
	    }

		public function init_post_type() 
		{
		    // get label
		    $labels = array(
		        'name' 					=> __('Memes Template', 'MG'),
		        'singular_name' 		=> __('meme', 'MG'),
		        'add_new' 				=> __('Add new meme', 'MG'),
		        'add_new_item' 			=> __('Add new meme', 'MG'),
		        'edit_item'			 	=> __('Edit meme', 'MG'),
		        'new_item' 				=> __('New meme', 'MG'),
		        'view_item' 			=> __('View meme', 'MG'),
		        'search_items' 			=> __('Search memes', 'MG'),
		        'not_found' 			=> __('No memes found', 'MG'),
		        'not_found_in_trash' 	=> __('No memes in trash', 'MG')
		    );
		   
		    // start formationg arguments
		    $args = array(
		        'labels' => $labels,
		        'public' => false,
		        'publicly_queryable' => true,
		        'show_ui' => true,
		        'has_archive' => true,
		        'query_var' => true,
		        'rewrite' => true,
		        'menu_icon' => MG_asset_url( 'images/icon1.png' ),
		        'capability_type' => 'post',
		        'show_in_menu' => true,
		        'supports' => array( 'title' )
		    );

		    register_post_type( 'memes_templates', $args );
			
			// add taxonomies for this post type
			$this->init_taxonomies();
			
			// add meta boxes to "memes" post type
			add_action('admin_menu', array($this, 'add_to_menu_metabox'));
			
			// change the layout of memes list
			add_filter('manage_edit-memes_templates_columns', array( $this, 'memes_edit_columns' ) );
			add_action('manage_posts_custom_column', array( $this, 'memes_posts_columns' ), 10, 2);  

			add_action( 'save_post', array( $this, 'save_post' ), 10, 3 );

			flush_rewrite_rules();
	    }

		// Initialize New Taxonomy Labels for the questions custom post type
		public function init_taxonomies() 
		{
			$labels = array(
				'name' 				=> __( 'memes Categories', 'MG' ),
				'singular_name' 	=> __( 'category', 'MG' ),
				'search_items' 		=> __( 'Search category Types', 'MG' ),
				'all_items' 		=> __( 'All category Types', 'MG' ),
				'parent_item' 		=> __( 'Parent category Types', 'MG' ),
				'parent_item_colon' => __( 'Parent category Types:', 'MG' ),
				'edit_item' 		=> __( 'Edit category Types', 'MG' ),
				'update_item' 		=> __( 'Update category Type', 'MG' ),
				'add_new_item' 		=> __( 'Add New category Type', 'MG' ),
				'new_item_name' 	=> __( 'New category Type', 'MG' ),
			);
			
		
		    // register taxonomy to hold our memecategs
		    register_taxonomy(
		        'memecategs',
				array('memes'),
		        array(
		            'hierarchical' => true,
		            'query_var' => true,
		            'show_in_nav_menus' => true,
		            'labels' => $labels,
		            'show_tagcloud' => false
		        )
		    );
		}

		public function save_post( $post_id, $post, $update )
		{
			// get the screen object to decide if we want to inject our scripts
			$screen = get_current_screen(); 

			if( isset($screen) && $screen->post_type == 'memes_templates' ){

				$mg_array = isset($_POST['MG']) ? $_POST['MG'] : array();
				if( count($mg_array) > 0 ){
					if( isset($mg_array['main_image']) && (int) $mg_array['main_image'] > 0 ){
						update_post_meta( $post_id, '_mg_main_image', $mg_array['main_image'] );
					}

					if( isset($mg_array['option']) && count($mg_array['option']) > 0 ){
						update_post_meta( $post_id, '_mg_options', $mg_array['option'] );
					}
					update_post_meta( $post_id, '_mg_used', $mg_array['used'] );
				}			}
		}

		public function add_to_menu_metabox()
		{
			// add meta box to "memes_templates" post type
			add_meta_box(
				$this->the_plugin->alias . '_meme_extra', 
				__('Extra meme', 'MG'), 
				array($this, 'meme_extra_fileds'), 
				'memes_templates', 
				'normal',
				'high'
			);
		}
		
		public function meme_extra_fileds()
		{
			global $post_id;
			
			$extra = array(
				'main_image'	=> get_post_meta( $post_id, "_mg_main_image", true ),
				'options'	=> get_post_meta( $post_id, "_mg_options", true ),
				'used'	=> (int)get_post_meta( $post_id, "_mg_used", true ),
			);
			$main_image = wp_get_attachment_url( $extra['main_image'] );

			if( $extra['used'] == 0 ){
				$extra['used'] = 1;
			}
			
			// debug
			if( 0 ) var_dump('<pre>',$extra,'</pre>');
			
			$html = array();
			$html[] = '<div class="MG_pt_meme_extra">';
			
			$html[] = 	'<div class="MG_configurator_wrapper">';
			$html[] = 		'<div class="MG_configurator_left_panel">';
			$html[] = 			'<div class="MG_configurator_main_image">';
			$html[] = 				'<div class="MG_configurator_main_image_wrapper">';
			if ( $main_image ) {
				$html[] = 				'<img src="' . ( $main_image ) . '" />';
			} 
			$html[] = 				'</div>';
			$html[] = 			'</div>';
			$html[] = 			'<label class="MG_tpl_used"><span>Template Used:</span> <input value="' . ( $extra['used'] ) . '" style="width: 80px" readonly type="text" name="MG[used]" /></label>';
			$html[] = 		'</div>';
			$html[] = 		'<div class="MG_configurator_right_panel">';
			$html[] = 			"<ul class='MG_options_text' data-options='" . ( json_encode($extra['options']) ) . "'>";
			$html[] = 				'<script type="text/template" id="MG_text-tpl-html">';

			$html[] = 				'<li class="option_unique_id">';
			$html[] = 					'<header>';
			$html[] = 						'<span>Text Box</span>';
			$html[] = 						'<a href="#" class="MG_delete_text"><i class="fa fa-trash" aria-hidden="true"></i></a>';
			$html[] = 						'<a href="#" class="MG_collapse_text"><i class="fa fa-minus" aria-hidden="true"></i></a>';
			$html[] = 					'</header>';
			$html[] = 					'<content>';
			$html[] = 						'<label>';
			$html[] = 							'<span>Width:</span>';
			$html[] = 							'<input type="text" name="MG[option][option_id][width]" class="MG-option-width" readonly /> <span>%</span>';
			$html[] = 						'</label>';
			$html[] = 						'<label>';
			$html[] = 							'<span>Height:</span>';
			$html[] = 							'<input type="text" name="MG[option][option_id][height]" class="MG-option-height" readonly /> <span>%</span>';
			$html[] = 						'</label>';
			$html[] = 						'<label>';
			$html[] = 							'<span>X position:</span>';
			$html[] = 							'<input type="text" name="MG[option][option_id][x]" class="MG-option-x" readonly /> <span>%</span>';
			$html[] = 						'</label>';
			$html[] = 						'<label>';
			$html[] = 							'<span>Y position:</span>';
			$html[] = 							'<input type="text" name="MG[option][option_id][y]" class="MG-option-y" readonly /> <span>%</span>';
			$html[] = 						'</label>';
			$html[] = 						'<label>';
			$html[] = 							'<span>Baseline:</span>';
			$html[] = 							'<select style="width: 70%" name="MG[option][option_id][text-position]"  class="MG-option-text-position">';
			$html[] = 								'<option value="top">Top</option>';
			$html[] = 								'<option value="bottom">Bottom</option>';
			$html[] = 							'</select>';
			$html[] = 						'</label>';
			$html[] = 						'<label>';
			$html[] = 							'<span>Placeholder:</span>';
			$html[] = 							'<input type="text" style="width: 70%" name="MG[option][option_id][placeholder]"  class="MG-option-placeholder" value="Text here" />';
			$html[] = 						'</label>';
			$html[] = 					'</content>';
			$html[] = 				'</li>';

			$html[] = 				'</script>';
			$html[] = 			'</ul>';
			$html[] = 			'<button class="MG_add_new_textbox">Create New Textbox</button>';
			$html[] = 			'<button class="MG_upload_image_btn">Upload Image</button>';
			$html[] = 		'</div>';
			$html[] = 	'</div>';

			$html[] = '</div>';
			
			echo implode( "\n", $html );
		}

		public function memes_edit_columns($columns) 
		{
		    $new_columns['cb'] 					= '<input type="checkbox" />';
		    $new_columns['meme_id'] 			= __('ID', 'MG');
		    $new_columns['meme_thumbnail'] 		= __('Image', 'MG');
		    $new_columns['title'] 				= __('Title', 'MG');
		    $new_columns['date'] 				= __('Date', 'MG');
		
		    return $new_columns;
		}
		
		public function memes_posts_columns($column_name, $id) 
		{
		    global $id;
		    switch ($column_name) {
				case 'meme_id':
		            echo $id;
		            break;
					
		        case 'meme_thumbnail':
		        	$main_image	= get_post_meta( $id, "_mg_main_image", true );
		        	$attachment_thumb = wp_get_attachment_image( $main_image, array(75, 75) );
 
		        	if( $attachment_thumb ){
		        		echo $attachment_thumb;
		        	}
		            break;
					
		        default:
		            break;
		    } // end switch
		}
    	
		public function extra_css() 
		{
			echo '<link href="'.$this->the_plugin->cfg["CURRENT_PT_URL"].'admin-memes-style.css" type="text/css" />';
		    echo "
		        <style type='text/css'>
		        th#avatar {width: 50px;}
		        th#meme_id {width: 30px;}
		        th#meme_thumbnail {width: 75px;}
		        th#meme_categs {width: 100px;}
		        th#meme_date {width: 100px;}
		        div.lcf-panel, div.lcf-panel-widget {box-shadow: none;}
		        </style>
			";
		}
	}
}
MGMemes::getInstance();