MemeGenerator = (function ($) {
    "use strict";


    var i18n = {
		settings: {
			'text-color': 'Text Color',
			'shadow-color': 'Outline Color',
		}
	};


    var maincontainer   = null,
    	mainimage 	   	= null,
    	mainimage_width	= 0,
		canvas			= null,
		context			= null,
		lightbox		= null,
		ajaxurl 		= null,
		pick_colors		= {},
		captions		= [],
		font_shadow		= null,
		template		= 0,
		watermark_text  = '',
		watermark_style  = {},
		min_cap_height  = 0,
		img 			= new Image();
    
    // init function, autoload
    (function init() {

        // load the triggers
        $(document).ready(function(){
            maincontainer = $(".mg-container");
            mainimage = maincontainer.find("#meme-generator-main-image");
            lightbox = maincontainer.find(".mg-lightbox");
            font_shadow = maincontainer.find("input[name='font-shadow']");

            triggers();
        });

        fill_up_colors();
    })();
	
	function fill_up_colors()
	{
		pick_colors['#ffffff'] = 'WHITE';
		pick_colors['#1abc9c'] = 'TURQUOISE';
		pick_colors['#2ecc71'] = 'EMERALD';
		pick_colors['#3498db'] = 'PETER RIVER';
		pick_colors['#9b59b6'] = 'AMETHYST';
		pick_colors['#34495e'] = 'WET ASPHALT';
		pick_colors['#16a085'] = 'GREEN SEA';
		pick_colors['#27ae60'] = 'NEPHRITIS';
		pick_colors['#2980b9'] = 'BELIZE HOLE';
		pick_colors['#8e44ad'] = 'WISTERIA';
		pick_colors['#DB0A5B'] = 'RAZZMATAZZ';
		pick_colors['#2c3e50'] = 'MIDNIGHT BLUE';
		pick_colors['#f1c40f'] = 'SUN FLOWER';
		pick_colors['#e67e22'] = 'CARROT';
		pick_colors['#e74c3c'] = 'ALIZARIN';
		pick_colors['#ecf0f1'] = 'CLOUDS';
		pick_colors['#95a5a6'] = 'CONCRETE';
		pick_colors['#f39c12'] = 'ORANGE';
		pick_colors['#d35400'] = 'PUMPKIN';
		pick_colors['#c0392b'] = 'POMEGRANATE';
		pick_colors['#bdc3c7'] = 'SILVER';
		pick_colors['#7f8c8d'] = 'ASBESTOS';
	}

	function set_template( id )
	{
		template = id;
	}

	function build_color_picker( elm, first )
	{
		var colors = $('<ul class="meme-generator-color-picker-list" />');
		$.each(pick_colors, function( key, value ){
			colors.append("<li style='background-color: " + ( key ) + "' title='" + ( value ) + "'></li>")
		});
		
		var picker = $( "<div class='meme-generator-color-picker' />" );
		picker.append( '<input type="hidden" />' );

		var current_color = $('<span class="meme-generator-current-color" />');
		current_color.css( "background-color", first );

		picker.append( current_color );
		
		picker.append(colors);
		elm.append( picker );

		colors.on('click', "li", function(){
			var that = $(this),
				parent = that.parents(".mg-text-box").eq(0),
				the_picker = that.parents("div").eq(0),
				color = that.css('background-color');

			the_picker.find(".meme-generator-current-color").css( "background-color", color );
			the_picker.find("input").val( color );

			var related_id = parent.attr('class').split(" ");
			related_id = related_id[1];
			
			update_caption_text( related_id, parent.find('input[type="text"]').val() )

			colors.hide();
		});

		picker.on('click', '.meme-generator-current-color', function(){

			maincontainer.find(".meme-generator-color-picker-list").hide();
			colors.show();
		});
	}

	function fitImageOn( canvas, context ) 
	{
		mainimage_width = mainimage.width();
		var imageAspectRatio = img.width / img.height;

		if( img.width > mainimage_width ){

			var renderableHeight = mainimage_width * img.height / img.width;
			$(canvas).attr( 'width', mainimage_width );
			$(canvas).attr( 'height', renderableHeight );

			mainimage.height( renderableHeight );

			context.drawImage(img, 0, 0, img.width, img.height, 0, 0, mainimage_width, renderableHeight );
		}
		else{
			var renderableHeight = mainimage_width * img.height / img.width;
			$(canvas).attr( 'width', mainimage_width );
			$(canvas).attr( 'height', renderableHeight );

			mainimage.height( renderableHeight );

			context.drawImage(img, 0, 0, img.width, img.height, 0, 0, mainimage_width, renderableHeight );
		}
	}

	function fix_max_font_size()
	{
		if( maincontainer.find('input[name="font-size"]').val() > min_cap_height ){
			min_cap_height = min_cap_height * 100 / 120;
			maincontainer.find('input[name="font-size"]').val( Math.floor( min_cap_height ) );
		}
	}

	function load_image( the_image )
	{
		img.src = the_image;
		img.onload = function() {
			canvas = mainimage.find("canvas")[0];

			context = canvas.getContext("2d");
			fitImageOn( canvas, context );

			$.each(captions, function(key, value ){
				add_text_caption( value );
			});

			fix_max_font_size();
		};
	}

	function imageLoader() 
	{
	    var reader = new FileReader();
	    reader.onload = function(event) {
	    	context.clearRect(0,0,canvas.width, canvas.height);
	    	load_image( this.result );
	    }
	    reader.readAsDataURL( maincontainer.find("#meme-generator-file-input")[0].files[0]);
	}

	function get_uniqid()
	{
		var n=Math.floor(Math.random()*11);
		var k = Math.floor(Math.random()* 1000000000);
		return $.trim( 'meme-generator-settings-' + $.trim( k ));
	}

	function add_text_caption( options )
	{
		var uniqid = get_uniqid();
		var settings_box = $("<div />");

		settings_box.addClass('mg-text-box');
		settings_box.addClass( uniqid );

		settings_box.append('<input type="text" placeholder="' + ( options.placeholder ) + '" />');

		var _opts_elm = $('<div class="mg-quick-options" />');

		var _font_color = $('<span class="mg-text-color" title="' + ( i18n.settings['text-color'] ) + '" />');
		_opts_elm.append( _font_color );

		var _shadow_color = $('<span class="mg-shadow-color" title="' + ( i18n.settings['shadow-color'] ) + '" />');
		_opts_elm.append( _shadow_color );

		_opts_elm.append( '<input type="number" value="10" name="shadow-spread" min="1" max="20">' );

		settings_box.append( _opts_elm );

		maincontainer.find('.mg-text-boxes').append(settings_box);

		build_color_picker( _font_color, options.colors.font );
		build_color_picker( _shadow_color, options.colors.shadow );


		var text_box = $("<div />");

		text_box.addClass('meme-generator-resizable');

		text_box.addClass( uniqid );
		text_box.data( 'box-id', uniqid );

		var handles_pos = [];
		handles_pos.push('nw');
		handles_pos.push('ne');
		handles_pos.push('sw');
		handles_pos.push('se');
		handles_pos.push('n');
		handles_pos.push('s');
		handles_pos.push('e');
		handles_pos.push('w');

		$.each(handles_pos, function(key, val){
			text_box.append( '<div class="ui-resizable-handle ui-resizable-' + ( val ) + '" />' );
		});

		text_box.data( "baseline", options['text-position'] );

		mainimage.append( text_box );
		make_resizable( text_box );

		var offset = 10;
		text_box.css({
			'width': mainimage_width - (offset * 2),
			'left': offset
		});

		var _canvas_holder = $('<div class="meme-generator-text-wrapper" />');

		if( options['pos'] ){
			text_box.css( {
				'top': options['pos']['y'] + "%",
				'left': options['pos']['x'] + "%",
				'width': options['pos']['width'] + "%",
				'height': options['pos']['height'] + "%"
			} );

			/*console.log({
				'top': options['pos']['y'] + "%",
				'left': options['pos']['x'] + "%",
				'width': options['pos']['width'] + "%",
				'height': options['pos']['height'] + "%"
			}  );*/
		}

		text_box.append( _canvas_holder );

		var _canvas_height = text_box.height();

		if( _canvas_height < min_cap_height || min_cap_height == 0 ){
			min_cap_height = _canvas_height;
		}
	}

	function update_caption_text( related_id, value, font_size )
	{
		var holder = maincontainer.find( ".meme-generator-resizable." + related_id );
		var settings = maincontainer.find( ".mg-text-box." + related_id );

		// default font size
		if( typeof(font_size) == "undefined" ){
			font_size = maincontainer.find('input[name="font-size"]').val();
		}

		var _canvas_holder = holder.find(".meme-generator-text-wrapper"),
			_canvas = $('<canvas />');

		var _ctx = _canvas[0].getContext("2d");


		var holder_height = holder.height();

		_canvas.attr( 'width', 9999);
		_canvas.attr( 'height', 9999 );
		
		_ctx.font = font_size + 'px ' + maincontainer.find(".mg-text-font-family").val();
		holder.data('font-size', font_size);
		var text_baseline = holder.data('baseline'),
			text_y = ( text_baseline == 'top' ? 0 : holder_height );

		_ctx.textBaseline = text_baseline;

		var blur = parseInt(settings.find("input[type='number']").val());
		var text_width = _ctx.measureText(value).width + blur * 2;

		text_width = Math.ceil( text_width );

		var h_width = holder.width();
		if( text_width >= h_width ){
			var extra_size_perc = text_width * 100 / (h_width - blur);

			var new_font_size = font_size * 100 / extra_size_perc;
			_ctx.font = new_font_size + 'px ' + maincontainer.find(".mg-text-font-family").val();
			text_width = h_width;
			holder.data('font-size', new_font_size);
		}

		if( !font_shadow.is(':checked') ){
			blur = 0;
		}
		_ctx.shadowColor = settings.find(".mg-shadow-color .meme-generator-current-color").css("background-color");
		_ctx.shadowOffsetX = text_width;
		_ctx.shadowOffsetY = text_y;
		_ctx.shadowBlur = blur;
		
		_ctx.fillText( value, -text_width + blur, 0);
		
		_ctx.fillStyle = settings.find(".mg-text-color .meme-generator-current-color").css("background-color");
		_ctx.shadowOffsetX = 0;
		_ctx.fillText( value, blur, text_y );

		_canvas_holder.css( 'width', text_width );
		_canvas_holder.css( 'height', holder.height() );
		_canvas_holder.html( _canvas );

		//_canvas.attr( 'width', text_width + 20 );
		//_canvas.attr( 'height', _canvas_holder.height() );

		holder.data( 'text', value );
		holder.data( 'width', text_width );
	}

	function set_ajaxurl( the_url )
	{
		ajaxurl = the_url;
	}

	function make_resizable( box )
	{
		box.draggable({
	      containment: mainimage
	    });

		box.resizable({
			containment: mainimage,
		    handles: {
		        'nw': '.ui-resizable-nw',
		        'ne': '.ui-resizable-ne',
		        'sw': '.ui-resizable-sw',
		        'se': '.ui-resizable-se',
		        'n': '.ui-resizable-n',
		        'e': '.ui-resizable-e',
		        's': '.ui-resizable-s',
		        'w': '.ui-resizable-w'
		    },
		    minHeight: 30,
		    resize: function( event, ui ) {
		    	var that = $(this),
		    		current_font_size = that.data("font-size"),
		    		related_id = that.data('box-id');

		    	if( typeof(current_font_size) == "undefined" ) {
		    		current_font_size = maincontainer.find('input[name="font-size"]').val();
		    	}
		    	var new_font_size = current_font_size;

		    	if( typeof(that.data( "height")) != "undefined" ){
			    	if( ui.size.height < that.data( "height") ) {

			    		if( current_font_size > ui.size.height ){
				    		if( typeof(that.data( 'orig-font-size')) == 'undefined' ){
					    		that.data( 'orig-font-size', new_font_size );
					    	}
				    		new_font_size = Math.floor( ui.size.height * 100 / 120 );
				    	}
			    	}
			    	else{
			    		if( ui.size.height < that.data( 'orig-font-size') ){
			    			new_font_size = Math.floor( ui.size.height * 100 / 120 );
			    		}else{
			    			new_font_size = that.data( 'orig-font-size');
			    		}
			    	}
			    }
		    	that.data( "height", ui.size.height );

			    update_caption_text( related_id, maincontainer.find(".mg-text-box." + related_id ).find('input[type="text"]').val(), new_font_size );
		    }
		});
	}

	function set_caption( caption )
	{
		captions.push( caption );
	}

	function generate_meme()
	{
		var image = canvas.toDataURL("image/png"),
			save_image = $("#mg-generate-test-img"),
			final_canvas = $("<canvas />");

		final_canvas.attr("width", $(canvas).width());
		final_canvas.attr("height", $(canvas).height());
		save_image.html(final_canvas);

		var final_context = final_canvas[0].getContext('2d');

		final_context.drawImage( canvas, 0, 0 );

		maincontainer.find(".meme-generator-resizable").each(function(){
			var that = $(this),
				_canvas = that.find("canvas"),
				text_wrapper = that.find(".meme-generator-text-wrapper"),
				_pos 	= that.position(),
				__pos = text_wrapper.position();

			var top = _pos.top + parseFloat(text_wrapper.css('margin-top'));
			var left = _pos.left + parseFloat(text_wrapper.css('margin-left'));
			
			if( typeof _canvas[0] != "undefined" ){
				final_context.drawImage( _canvas[0], left, top );
			}
		});

		// print watermark
		var watermark_canvas = $("<canvas width='9999' height='9999' />"),
			watermark_context = watermark_canvas[0].getContext('2d');

		watermark_context.font = watermark_style.weight + " " + watermark_style.size + "px '" + watermark_style.family + "'";
	    watermark_context.textBaseline = 'top';
	    watermark_context.strokeStyle = 'black';
	    watermark_context.lineWidth = 1;

	    var _top_pos = $(canvas).height() - 40;
	    watermark_context.strokeText( watermark_text, 0, 10 );

	   	watermark_context.fillStyle = watermark_style.color;
	    watermark_context.fillText( watermark_text, 0, 10 );

		final_context.drawImage( watermark_canvas[0], 10, _top_pos );
		
		//return ;

		$.post(
		    ajaxurl, 
		    {
		        'action'		: 'mg_ajax_action',
		        'sub_action'	: 'save_meme',
		        'template'		: template,
		        'image'			: final_canvas[0].toDataURL('image/png')
		    }, 
		    function( response ){
		        if( response.status == 'valid' ){
		        	window.location = response.redirect_url;
		        	return true;
		        }
		    },
		    'json'
		);
	}

	function set_watermark( new_text )
	{
		watermark_text = new_text;
	}

	function set_watermark_style( obj )
	{
		watermark_style = $.parseJSON( obj );
	}

	function view_count( that )
	{
		var id = that.data('id');

		var data = {
			'action': 'MG_pt_meme_ajax_requests',
			'sub_action': 'admin_view_statistics',
			'post_id': id
		};

		$.post(ajaxurl, data, function(response) {
			if( response.status == 'valid' ){
				var totals = 0;
				$.each( response.data, function(key, value){
					totals += value;
				});

				that.replaceWith( totals );
			}
		}, 'json');
	}

	function load_memes( paged )
	{
		var data = {
			'action': 'MG_pt_meme_ajax_requests',
			'sub_action': 'load_memes',
			'paged': paged
		};

		lightbox.find(".mg-lightbox-wrapper .mg-lightbox-memes-lists").css('opacity', '0.5');
		lightbox.find(".mg-lightbox-loading").show();

		$.post(ajaxurl, data, function(response) {
			if( response.status == 'valid' ){
				lightbox.find(".mg-lightbox-wrapper .mg-lightbox-memes-lists").html( response.html );
				lightbox.find(".mg-lightbox-wrapper .mg-lightbox-memes-navs").html( response.nav );
				lightbox.find(".mg-lightbox-wrapper .mg-lightbox-memes-lists").css('opacity', 1);

				lightbox.find(".mg-lightbox-loading").hide();
			}
		}, 'json');
	}

    function triggers() 
    {
		maincontainer.on('click', '#mg-upload-own-image', function(){
			maincontainer.find("#meme-generator-file-input").click();
		});

		maincontainer.on('click', '#mg-action-generate-meme', function(){
			generate_meme();
		});

		maincontainer.on('click', ".mg-lightbox-memes-navs a", function(e){
			e.preventDefault();

			var that = $(this);

			if( !that.hasClass('on') ){
				load_memes( that.data('nav') );
			}
		});

		maincontainer.on( 'change', "#meme-generator-file-input", imageLoader );

		$('body').on("keyup", '.mg-text-box input[type="text"]', function(){
			var related_id = $(this).parent().attr('class').split(" ");
			related_id = related_id[1];

			update_caption_text( related_id, $(this).val() );
		});

		maincontainer.on('change', 'input[name="shadow-spread"]', function(){
			var that = $(this),
				parent = that.parents(".mg-text-box").eq(0);

			var related_id = parent.attr('class').split(" ");
			related_id = related_id[1];
			
			update_caption_text( related_id, parent.find('input[type="text"]').val() );
		});

		maincontainer.on('click', 'a.mg-view-all-btn', function(e){
			e.preventDefault();

			var that = $(this);
			lightbox.fadeIn(200);

			load_memes( 1 );
		});
		
		// Close Lightbox Button
		maincontainer.on('click', '.mg-close-lightbox', function(){
			lightbox.fadeOut(200);
		});

		// Close Lightbox if closed outside the box
		lightbox.mouseup(function (e) {
			var lightboxContent = $(".mg-lightbox-content");
			if (!lightboxContent.is(e.target) && lightboxContent.has(e.target).length === 0)
			{
				lightbox.fadeOut(200);
			}
		});

		// Hide options toggle

		var	originalValue = $("#mg-options span").text(),
			newValue = $("#mg-options span").attr('data-switch');

		$("#mg-options").click(function() {
			var currentValue = $(this).find('span').text(),
				currentItemI = $(this).find('i'),
				currentItem = $(this).find('span');

			if ( currentValue == originalValue ) {
				currentItem.text(newValue)
				currentItemI.removeClass('fa-caret-up').addClass('fa-caret-down');
			} else {
				currentItem.text(originalValue);
				currentItemI.removeClass('fa-caret-down').addClass('fa-caret-up');
			}

			$( ".mg-text-options" ).slideToggle("fast");
		});

		// Initiate Carousel
		$("#mg-carousel").owlCarousel({
			items: 6,
			pagination: false,
			navigation: true,
			itemsDesktop : [1199,6],
			itemsDesktopSmall : [1024,6],
			itemsTablet: [768,4],
			itemsTabletSmall: false,
			itemsMobile : [479,4],
			navigationText: ["&#xf104","&#xf105"]
		});

		$("#mg-carousel-related").owlCarousel({
			items: 4,
			loop:true,
			pagination: false,
			navigation: true,
			itemsDesktop : [1199,4],
			itemsDesktopSmall : [1024,4],
			itemsTablet: [768,3],
			itemsTabletSmall: false,
			itemsMobile : [479,2],
			navigationText: ["&#xf104","&#xf105"]
		}).trigger('initialized.owl.carousel').show();

		$("#mg-carousel").find(".owl-buttons").append('<a href="#" class="mg-view-all-btn">View all memes templates</a>');
		
		maincontainer.on('change', 'select.mg-text-font-family,input[name="font-size"]', function(){
			var that = $(this);

			var settings_elms = $(".mg-text-boxes .mg-text-box");

			settings_elms.each(function(){
				var parent = $(this),
					related_id = parent.attr('class').split(" ");
				related_id = related_id[1];
					
				update_caption_text( related_id, parent.find('input[type="text"]').val() );
			});
		});

		maincontainer.on('click', '#mg-add-another-box', function(){
			var that = $(this);

			add_text_caption({
				'pos'			: {
					'x': '5',
					'y': '40',
					'width': '90',
					'height': '20'
				},
				'placeholder'	: 'New Text',
				'colors'		: {
					'font'	: '#ffffff',
					'shadow': '#e67e22'
				}
			});
		});

		maincontainer.on("click", ".mg-carousel-holder .item, .mg-lightbox-memes-lists div", function(){
			var that = $(this),
				url = that.data('url'),
				title = that.data('title'),
				id = that.data('template'),
				options = that.data('options');

			if( typeof(title) == 'undefined' ){
				title = that.attr('title');
			}

			captions = [];
			mainimage.find(".meme-generator-resizable").remove();
			maincontainer.find(".mg-text-box").remove();

			$.each(options, function(key, value){
				set_caption( value );
			});

			load_image( url );
			set_template( id );

			maincontainer.find(".mg-meme-name h4").text( title );
			lightbox.fadeOut(200);
		})

		$('body').find("#mg-total-shares").each(function(){
			view_count( $(this) );
		});

		$('body').on("click", "#mg-show-embed-code", function(e){
			e.preventDefault();

			$(".mg-embed-codes").toggle();
		});

		$("body").on("change", 'input[name="all-caps"]', function(){
			var that = $(this);

			var settings_elms = $(".mg-text-boxes .mg-text-box");

			settings_elms.each(function(){
				var parent = $(this),
					related_id = parent.attr('class').split(" ");

				related_id = related_id[1];
				

				if( that.is(':checked') ){
					update_caption_text( related_id, parent.find('input[type="text"]').val().toUpperCase() );
				}else{
					update_caption_text( related_id, parent.find('input[type="text"]').val().toLowerCase() );
				}
			});
		});

		$("body").on("change", 'input[name="font-shadow"]', function(){
			var that = $(this);

			var settings_elms = $(".mg-text-boxes .mg-text-box");

			settings_elms.each(function(){
				var parent = $(this),
					related_id = parent.attr('class').split(" ");

				related_id = related_id[1];
				
				update_caption_text( related_id, parent.find('input[type="text"]').val());
			});
		});

		$('body').on("click", "#mg-reset-options", function(e){
			e.preventDefault();

			if( confirm("Are you sure you want to reset?") ){
				window.location.reload();
				return true;
			}
		});
		
    }

	// external usage
	return {
		"load_image"			: load_image,
		"set_template"			: set_template,
		"set_watermark"			: set_watermark,
		"set_watermark_style" 	: set_watermark_style,
		"set_ajaxurl"			: set_ajaxurl,
		"set_caption"			: set_caption,
		"view_count"			: view_count 
	}
})(jQuery);